"use strict";
/**
 * penguins-eggs
 * command: syncto.ts
 * author: Piero Proietti
 * email: piero.proietti@gmail.com
 * license: MIT
 */
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
// https://gist.github.com/ansemjo/6f1cf9d9b8f7ce8f70813f52c63b74a6
const core_1 = require("@oclif/core");
const fs_1 = tslib_1.__importDefault(require("fs"));
const utils_1 = require("../lib/utils");
const compressors_1 = tslib_1.__importDefault(require("../classes/compressors"));
const settings_1 = tslib_1.__importDefault(require("../classes/settings"));
const utils_2 = tslib_1.__importDefault(require("../classes/utils"));
/**
 *
 */
class Syncto extends core_1.Command {
    constructor() {
        super(...arguments);
        this.verbose = false;
        this.echo = {};
        this.luksName = 'luks-volume';
        this.luksFile = `/tmp/${this.luksName}`;
        this.luksDevice = `/dev/mapper/${this.luksName}`;
        this.luksMountpoint = `/tmp/mnt/${this.luksName}`;
        this.privateSquashfs = `private.squashfs`;
        this.excludeHome = '/etc/penguins-eggs.d/exclude.list.d/home.list';
        this.exclude = false;
        this.settings = {};
    }
    /**
     *
     */
    async run() {
        utils_2.default.titles(this.id + ' ' + this.argv);
        const { flags } = await this.parse(Syncto);
        if (flags.verbose) {
            this.verbose = true;
        }
        this.echo = utils_2.default.setEcho(this.verbose);
        let fileLuks = '';
        if (flags.file) {
            fileLuks = flags.file;
        }
        if (flags.excludes) {
            this.exclude = true;
        }
        if (utils_2.default.isRoot()) {
            await this.luksCreate();
        }
        else {
            utils_2.default.useRoot(this.id);
        }
    }
    /**
     *
     */
    async luksCreate() {
        await (0, utils_1.exec)(`rm -rf ${this.luksFile}`);
        //==========================================================================
        // Create squashfs
        //==========================================================================
        utils_2.default.warning(`Creating private.squashfs`);
        const compressors = new compressors_1.default();
        await compressors.populate();
        // comp
        let comp = '';
        this.settings = new settings_1.default();
        if (await this.settings.load()) {
            let compression = compressors.fast();
            if (this.settings.config.compression == `max`) {
                compression = compressors.max();
            }
            else if (this.settings.config.compression == `standard`) {
                compression = compressors.standard();
            }
            comp = `-comp ${compression}`;
        }
        // exclude /home/eggs
        let exclude_nest = '';
        if (fs_1.default.existsSync(this.settings.work_dir.workdir)) {
            exclude_nest = `-e ${this.settings.config.snapshot_dir}`;
        }
        // exclude file
        let exclude_home = '';
        if (this.exclude) {
            exclude_home = `-ef ${this.excludeHome}`;
        }
        // creato dummyfs per /etc/
        await (0, utils_1.exec)(`mkdir -p /tmp/dummyfs/etc`, this.echo);
        await (0, utils_1.exec)(`cp -a /etc/passwd /tmp/dummyfs/etc`, this.echo);
        await (0, utils_1.exec)(`cp -a /etc/group /tmp/dummyfs/etc`, this.echo);
        await (0, utils_1.exec)(`cp -a /etc/shadow /tmp/dummyfs/etc`, this.echo);
        await (0, utils_1.exec)(`mkdir -p /tmp/dummyfs/etc/lightdm`, this.echo); // lightdm
        await (0, utils_1.exec)(`cp -a /etc/lightdm/lightdm.conf /tmp/dummyfs/etc/lightdm/`, this.echo); // lightdm  
        let mkPrivateSquashfs = `mksquashfs \
                              /tmp/dummyfs/etc \
                              /home \
                              /tmp/${this.privateSquashfs} \
                              ${comp} \
                              ${exclude_nest} \
                              ${exclude_home} \
                              -keep-as-directory \ 
                              -noappend`;
        mkPrivateSquashfs = mkPrivateSquashfs.replace(/\s\s+/g, ` `);
        await (0, utils_1.exec)(mkPrivateSquashfs, utils_2.default.setEcho(true));
        // remove dummyfs
        await (0, utils_1.exec)(`rm /tmp/dummyfs/ -rf`, this.echo);
        //==========================================================================
        // Create LUKS volume
        //==========================================================================
        utils_2.default.titles("Creating LUKS Volume");
        // calcolo size
        let sizeString = (await (0, utils_1.exec)(`unsquashfs -s /tmp/${this.privateSquashfs} | grep "Filesystem size" | sed -e 's/.*size //' -e 's/ .*//'`, { echo: false, capture: true })).data;
        let size = parseInt(sizeString) + 2048;
        console.log("size private.squashfs:", bytesToGB(size), size);
        let luksBlockSize = 512;
        let luksBlocks = Math.ceil(size / luksBlockSize);
        size = luksBlockSize * luksBlocks;
        // Aggiungo un 20% in più per ottenere luksSize
        let luksSize = Math.ceil((size) * 1.20);
        console.log("luksSize:", bytesToGB(luksSize), luksSize);
        // truncate * 2048 è cruciale
        let truncateAt = luksSize * 2048;
        utils_2.default.warning(`Preparing file ${this.luksFile} for ${this.luksDevice}, size ${truncateAt}`);
        await (0, utils_1.exec)(`truncate --size ${luksSize} ${this.luksFile}`, this.echo);
        utils_2.default.warning(`Creating LUKS Volume on ${this.luksFile}`);
        await (0, utils_1.exec)(`cryptsetup --batch-mode luksFormat ${this.luksFile}`, utils_2.default.setEcho(true));
        console.log('');
        // open LUKS volume temp
        utils_2.default.warning(`Opening LUKS Volume on ${this.luksFile}`);
        let code = (await (0, utils_1.exec)(`cryptsetup luksOpen ${this.luksFile} ${this.luksName}`, utils_2.default.setEcho(true))).code;
        if (code != 0) {
            utils_2.default.error(`cryptsetup luksOpen ${this.luksFile} ${this.luksName} failed`);
            process.exit(code);
        }
        await (0, utils_1.exec)('udevadm settle', this.echo);
        // formatta ext4 il volume
        await (0, utils_1.exec)(`mkfs.ext4 ${this.luksDevice}`, this.echo);
        console.log('');
        // mount LUKS volume
        if (!fs_1.default.existsSync(`this.luksMountpoint)`)) {
            utils_2.default.warning(`creating mountpoint ${this.luksMountpoint}`);
            await (0, utils_1.exec)(`mkdir -p ${this.luksMountpoint}`, this.echo);
        }
        if (!utils_2.default.isMountpoint(`${this.luksMountpoint}`)) {
            utils_2.default.warning(`mounting volume: ${this.luksDevice} on ${this.luksMountpoint}`);
            let code = (await (0, utils_1.exec)(`mount ${this.luksDevice} ${this.luksMountpoint}`, utils_2.default.setEcho(true))).code;
            if (code != 0) {
                utils_2.default.error(`mount ${this.luksDevice} ${this.luksMountpoint} failed`);
                process.exit(code);
            }
        }
        // copy private.squashfs
        utils_2.default.warning(`moving /tmp/${this.privateSquashfs} to ${this.luksMountpoint}`);
        await (0, utils_1.exec)(`mv /tmp/${this.privateSquashfs} ${this.luksMountpoint}`, this.echo);
        utils_2.default.warning(`Umounting ${this.luksMountpoint}`);
        await (0, utils_1.exec)(`umount ${this.luksMountpoint}`, this.echo);
        utils_2.default.warning(`Vlosing ${this.luksMountpoint}`);
        await (0, utils_1.exec)(`cryptsetup luksClose ${this.luksName}`, this.echo);
    }
}
Syncto.flags = {
    file: core_1.Flags.string({ char: 'f', description: 'file luks-volume encrypted' }),
    excludes: core_1.Flags.boolean({ char: 'e', description: 'use: exclude.list.d/home.list' }),
    help: core_1.Flags.help({ char: 'h' }),
    verbose: core_1.Flags.boolean({ char: 'v', description: 'verbose' }),
};
Syncto.description = 'Save users and users\' data ENCRYPTED';
Syncto.examples = [
    'sudo eggs syncto',
    'sudo eggs syncto --file /path/to/luks-volume',
    'sudo eggs syncto --excludes'
];
exports.default = Syncto;
/**
 * Convert bytes to gigabytes
 */
function bytesToGB(bytes) {
    const gigabytes = bytes / 1073741824;
    return gigabytes.toFixed(2) + ' GB';
}
