"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
* converter
*/
const fs_1 = tslib_1.__importDefault(require("fs"));
const mustache_1 = tslib_1.__importDefault(require("mustache"));
const js_yaml_1 = tslib_1.__importDefault(require("js-yaml"));
const unified_1 = tslib_1.__importDefault(require("unified"));
const to_vfile_1 = require("to-vfile");
const remark_man_1 = tslib_1.__importDefault(require("remark-man"));
const remark_parse_1 = tslib_1.__importDefault(require("remark-parse"));
const remark_rehype_1 = tslib_1.__importDefault(require("remark-rehype"));
const rehype_document_1 = tslib_1.__importDefault(require("rehype-document"));
const rehype_format_1 = tslib_1.__importDefault(require("rehype-format"));
const rehype_stringify_1 = tslib_1.__importDefault(require("rehype-stringify"));
const utils_1 = require("../lib/utils");
const utils_2 = tslib_1.__importDefault(require("./utils"));
/**
 * class Man
 */
class Converter {
    constructor(readmeName = './README.md') {
        this.readmeName = '';
        this.readmeName = readmeName;
    }
    /**
     * readme2md
     */
    async readme2md(verbose = false) {
        if (verbose) {
            console.log('Converter.readme2md()');
        }
        if (fs_1.default.existsSync('pb.yaml')) {
            let pbPackage = {};
            pbPackage = js_yaml_1.default.load(fs_1.default.readFileSync('pb.yaml', 'utf-8'));
            const readme = fs_1.default.readFileSync(this.readmeName, 'utf-8').split('\n');
            /**
             * Sezioni: toc, usage, commands
             */
            let toc = '';
            const tocStart = '<!-- toc -->';
            let isToc = false;
            const tocStop = '<!-- tocstop -->';
            let usage = '';
            const usageStart = '<!-- usage -->';
            let isUsage = false;
            const usageStop = '<!-- usagestop -->';
            let commands = '';
            const commandsStart = '<!-- commands -->';
            let isCommands = false;
            const commandsStop = '<!-- commandsstop -->';
            for (let i = 0; i < readme.length; i++) {
                let isComment = false;
                if (readme[i].includes('<!--')) {
                    isComment = false;
                    if (readme[i].includes(tocStart)) {
                        isToc = true;
                    }
                    if (readme[i].includes(tocStop)) {
                        isToc = false;
                    }
                    if (readme[i].includes(usageStart)) {
                        isUsage = true;
                    }
                    if (readme[i].includes(usageStop)) {
                        isUsage = false;
                    }
                    if (readme[i].includes(commandsStart)) {
                        isCommands = true;
                    }
                    if (readme[i].includes(commandsStop)) {
                        isCommands = false;
                    }
                }
                // Aggiunge la linea alla sezione
                if (isToc && !isComment) {
                    toc += readme[i] + '\n';
                }
                if (isUsage && !isComment) {
                    usage += readme[i] + '\n';
                }
                if (isCommands && !isComment) {
                    if (!readme[i].includes('See code:')) {
                        commands += readme[i] + '\n';
                    }
                }
            }
            toc = '';
            usage = usage.toString();
            /**
            * Creazione della versione markdown
            */
            const tempMd = pbPackage.destDir + '/DEBIAN/' + pbPackage.name + '.md';
            const template = fs_1.default.readFileSync('perrisbrewery/template/man.template.md', 'utf8');
            const sourceVersion = pbPackage.sourceVersion;
            let linuxVersion = 'linux-x32';
            if (pbPackage.linuxArch === 'amd64') {
                linuxVersion = 'linux-x64';
            }
            else if (pbPackage.linuxArch === 'arm64') {
                linuxVersion = 'linux-arm64';
            }
            else if (pbPackage.linuxArch === 'armel') {
                linuxVersion = 'linux-arm';
            }
            const nodeVersion = pbPackage.nodeVersion;
            const view = {
                toc: toc,
                usage: usage,
                commands: commands,
                sourceVersion: sourceVersion,
                linuxVersion: linuxVersion,
                nodeVersion: nodeVersion
            };
            fs_1.default.writeFileSync(tempMd, mustache_1.default.render(template, view), 'utf8');
        }
    }
    /**
     * md2man
     */
    async md2man(verbose = false) {
        // const { read, write } = await import('to-vfile')
        // const { unified } = await import('unified')
        // const { default: remarkMan } = await import('remark-man')
        // const { default: remarkParse } = await import('remark-parse')
        const echo = utils_2.default.setEcho(verbose);
        if (verbose) {
            console.log('Converter.md2man()');
        }
        if (fs_1.default.existsSync('pb.yaml')) {
            let pbPackage = {};
            pbPackage = js_yaml_1.default.load(fs_1.default.readFileSync('pb.yaml', 'utf8'));
            const source = pbPackage.destDir + '/DEBIAN/' + pbPackage.name + '.md';
            let dirname = pbPackage.destDir + '/DEBIAN/';
            const basename = pbPackage.name;
            const extname = '.roll';
            const file = await (0, unified_1.default)()
                .use(remark_parse_1.default)
                .use(remark_man_1.default)
                .process(await (0, to_vfile_1.read)(source));
            file.dirname = dirname;
            file.basename = basename;
            file.extname = extname;
            await (0, to_vfile_1.write)(file);
            const man = dirname + basename + extname;
            const manCompressed = dirname + basename + '.roll.gz';
            await (0, utils_1.exec)('gzip -9 ' + man, echo);
            if (pbPackage.name === 'eggs') {
                pbPackage.name = 'penguins-eggs';
            }
            await (0, utils_1.exec)(`cp ${manCompressed} ${pbPackage.destDir}/usr/lib/${pbPackage.name}/manpages/doc/man/${basename}.1.gz`, echo);
        }
    }
    /**
     * md2html
     */
    async md2html(verbose = false) {
        // const { read, write } = await import('to-vfile')
        // const { default: remarkParse } = await import('remark-parse')
        // const { default: remarkRehype } = await import('remark-rehype')
        // const { default: rehypeDocument } = await import('rehype-document')
        // const { default: rehypeFormat } = await import('rehype-format')
        // const { default: rehypeStringify } = await import('rehype-stringify')
        const echo = utils_2.default.setEcho(verbose);
        if (verbose) {
            console.log('Converter.md2html()');
        }
        if (fs_1.default.existsSync('pb.yaml')) {
            let pbPackage = {};
            pbPackage = js_yaml_1.default.load(fs_1.default.readFileSync('pb.yaml', 'utf8'));
            const source = pbPackage.destDir + '/DEBIAN/' + pbPackage.name + '.md';
            let dirname = pbPackage.destDir + '/usr/lib/' + pbPackage.name + '/manpages/doc/man/';
            if (pbPackage.name === 'eggs') {
                dirname = pbPackage.destDir + '/usr/lib/penguins-eggs/manpages/doc/man/';
            }
            const basename = pbPackage.name;
            const extname = '.html';
            const file = await (0, unified_1.default)()
                .use(remark_parse_1.default)
                .use(remark_rehype_1.default)
                .use(rehype_document_1.default)
                .use(rehype_format_1.default)
                .use(rehype_stringify_1.default)
                .process(await (0, to_vfile_1.read)(source));
            file.dirname = dirname;
            file.basename = basename;
            file.extname = extname;
            await (0, to_vfile_1.write)(file);
            //await exec(`mv ${dirname}${basename}${extname} ${dirname}${basename}.1.html`, echo)
        }
    }
}
exports.default = Converter;
