"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * perri's brewery
 *
 * Class: dpkg
 *
 */
const fs_1 = tslib_1.__importDefault(require("fs"));
const mustache_1 = tslib_1.__importDefault(require("mustache"));
const path_1 = tslib_1.__importDefault(require("path"));
const utils_1 = tslib_1.__importDefault(require("./utils"));
const dependencies_1 = require("../lib/dependencies");
const utils_2 = require("../lib/utils");
/**
 * class Dpkg
 * analyze
 */
class Dpkg {
    constructor() {
        this.pbPackage = {};
    }
    /**
     * Receive a file .deb or a path (structured as file deb)
     * @param debPackage
     * @return IPackage
     */
    async analyze(pathSource = '', verbose = false) {
        if (verbose) {
            console.log('dpkg.analyze()');
        }
        this.pbPackage.path = path_1.default.dirname(pathSource) + '/';
        const debPackage = path_1.default.basename(pathSource);
        const names = debPackage.split('_'); // eggs 9.0.0 amd64.deb
        this.pbPackage.name = names[0]; // eggs
        this.pbPackage.sourceVersion = names[1]; // 9.0.0
        this.pbPackage.linuxArch = names[2].substring(0, names[2].indexOf(".")); // amd64
        this.pbPackage.linuxArch = 'i386';
        if (debPackage.includes('amd64')) {
            this.pbPackage.linuxArch = 'amd64';
        }
        else if (debPackage.includes('arm64')) {
            this.pbPackage.linuxArch = 'arm64';
        }
        else if (debPackage.includes('armel')) {
            this.pbPackage.linuxArch = 'armel';
        }
        this.pbPackage.tempDir = `./perrisbrewery/workdir/${this.pbPackage.name}_${this.pbPackage.sourceVersion}_${this.pbPackage.linuxArch}`;
        this.pbPackage.nodeVersion = process.version;
        this.pbPackage.destDir = this.pbPackage.tempDir;
        return this.pbPackage;
    }
    /**
     *
     */
    async pack(pbPackage, verbose = false) {
        const echo = utils_1.default.setEcho(verbose);
        if (verbose) {
            console.log('dpkg.pack()');
        }
        this.pbPackage = pbPackage;
        await (0, utils_2.exec)(`dpkg-deb --build ${this.pbPackage.destDir}`, echo);
    }
    /**
     * unpack
     */
    async unpack(verbose = false) {
        const echo = utils_1.default.setEcho(verbose);
        if (verbose) {
            console.log('dpkg.unpack()');
        }
        if (fs_1.default.existsSync(this.pbPackage.tempDir)) {
            await (0, utils_2.exec)(`rm ${this.pbPackage.tempDir} -rf`, echo);
        }
        await (0, utils_2.exec)(`mkdir ${this.pbPackage.tempDir} `, echo);
        await (0, utils_2.exec)(`dpkg-deb -R ${this.pbPackage.path}${this.pbPackage.name}_${this.pbPackage.sourceVersion}_${this.pbPackage.linuxArch}.deb ${this.pbPackage.tempDir}`, echo);
    }
    /**
     * copy preinst, postinst, prerm and postrm
     */
    async makeScripts(verbose = false) {
        const echo = utils_1.default.setEcho(verbose);
        if (verbose) {
            console.log('dpkg.makeScripts()');
        }
        await (0, utils_2.exec)(`cp ./perrisbrewery/scripts/* ${this.pbPackage.destDir}/DEBIAN/`, echo);
    }
    /**
     * makeControl
     * Add dependencies common
     */
    makeControl(verbose = false) {
        let packages = dependencies_1.depCommon;
        if (verbose) {
            console.log('dpkg.makeControl()');
        }
        const arch = utils_1.default.machineArch();
        dependencies_1.depArch.forEach((dep) => {
            if (dep.arch.includes(arch)) {
                packages.push(dep.package);
            }
        });
        packages.sort();
        const depends = (0, dependencies_1.array2comma)(packages);
        const template = fs_1.default.readFileSync('perrisbrewery/template/control.template', 'utf8');
        const view = {
            name: this.pbPackage.name,
            version: this.pbPackage.sourceVersion,
            section: 'main',
            priority: 'standard',
            arch: this.pbPackage.linuxArch,
            mantainer: 'artisan <piero.proietti@gmail.com>',
            description: 'Perri\'s Brewery edition',
            depends: depends,
        };
        // depends, suggest e conflicts vengono gestiti a mano
        fs_1.default.writeFileSync(`${this.pbPackage.destDir}/DEBIAN/control`, mustache_1.default.render(template, view));
    }
}
exports.default = Dpkg;
