"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const js_yaml_1 = tslib_1.__importDefault(require("js-yaml"));
const utils_1 = tslib_1.__importDefault(require("../classes/utils"));
const dir_1 = tslib_1.__importDefault(require("../classes/dir"));
const dpkg_1 = tslib_1.__importDefault(require("../classes/dpkg"));
const converter_1 = tslib_1.__importDefault(require("../classes/converter"));
const utils_2 = require("../lib/utils");
/**
 *
 */
class Deb extends core_1.Command {
    async run() {
        const { args, flags } = await this.parse(Deb);
        const verbose = flags.verbose;
        const echo = utils_1.default.setEcho(verbose);
        utils_1.default.titles(this.id + ' ' + this.argv);
        let pbPackage = {};
        const here = process.cwd() + '/';
        let pathSource = here;
        if (args.pathSource !== undefined) {
            if (pathSource.substring(pathSource.length, -1) !== '/') {
                pathSource = args.pathSource + '/';
            }
            else {
                pathSource = args.pathSource;
            }
        }
        if (!fs_1.default.existsSync(`${here}/perrisbrewery`)) {
            fs_1.default.mkdirSync(`${here}/perrisbrewery`);
            await (0, utils_2.exec)(`cp -r ${path_1.default.resolve(__dirname, `../perrisbrewery/template`)} ${here}/perrisbrewery`, echo);
            await (0, utils_2.exec)(`cp -r ${path_1.default.resolve(__dirname, `../perrisbrewery/scripts`)} ${here}/perrisbrewery`, echo);
            console.log('perrisbrewery dir created in: ' + pathSource);
            console.log('Edit configuration in template e scripts. Include /perribrewery/workdir in your .gitignore.');
            console.log('After sudo npm run deb (build deb package with oclif');
            console.log('Finally run pb to rebuild your packages with manual, scripts, etc');
            process.exit(0);
        }
        /**
         * remove workdir if exist and create a new one
         */
        if (fs_1.default.existsSync(`${here}/perrisbrewery/workdir`)) {
            await (0, utils_2.exec)(`rm ${here}/perrisbrewery/workdir -rf`, echo);
        }
        await (0, utils_2.exec)(`mkdir ${here}/perrisbrewery/workdir`, echo);
        await (0, utils_2.exec)(`touch ${here}/perrisbrewery/workdir/.gitkeep`, echo);
        this.log('-pathSource: ' + pathSource);
        const dpkg = new dpkg_1.default();
        const dir = new dir_1.default();
        const packagesNames = await dir.analyze(pathSource, verbose);
        this.log('-packagesNames: ' + packagesNames);
        /**
         * inizio il lood fra  i pacchetti
         */
        const converter = new converter_1.default(pathSource + '/README.md');
        for (const packageName of packagesNames) {
            this.log('- package: ' + packageName);
            pbPackage = await dpkg.analyze(pathSource + 'dist/deb/' + packageName, verbose);
            fs_1.default.writeFileSync('pb.yaml', js_yaml_1.default.dump(pbPackage), 'utf-8');
            /**
             * unpack
             */
            await dpkg.unpack(verbose);
            // insert script and control
            await dpkg.makeScripts(verbose);
            dpkg.makeControl(verbose);
            // create md, man and html from README.md
            await converter.readme2md(verbose);
            await converter.md2man(verbose);
            await converter.md2html(verbose);
            // pack in deb
            await dpkg.pack(pbPackage, verbose);
            // remove tempDir
            if (!flags.mantain) {
                await (0, utils_2.exec)(`rm ${pbPackage.tempDir} -rf`, echo);
            }
        }
    }
}
exports.default = Deb;
Deb.description = 'add preinst, postinst, prerm, postrm scripts and manPages to .deb';
Deb.flags = {
    verbose: core_1.Flags.boolean({ char: 'v', description: 'verbose' }),
    help: core_1.Flags.help({ char: 'h' }),
    mantain: core_1.Flags.boolean({ char: 'm' }),
};
Deb.args = [{ name: 'pathSource' }];
